'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Kinet = function () {
    function Kinet(options) {
        _classCallCheck(this, Kinet);

        this._handlers = {
            start: [],
            tick: [],
            end: []
        };

        this._friction = options && options.friction ? 1 - options.friction : 1 - 0.3;
        this._acceleration = options && options.acceleration ? options.acceleration : 0.04;
        this._raf = null;
        this._target = options && options.initialValue ? options.initialValue : 0;

        this.current = options && options.initialValue ? options.initialValue : 0;
        this.velocity = 0;
    }

    _createClass(Kinet, [{
        key: 'animate',
        value: function animate() {
            var _this = this;

            var distance = this.update();

            if (Math.abs(this.current - this.target) > 0.1) {
                this._raf = requestAnimationFrame(this.animate.bind(this));
                this._handlers['tick'].forEach(function (handler) {
                    return handler(_this.current, _this.target);
                });
            } else {
                this.current = this.target;
                this._handlers['end'].forEach(function (handler) {
                    return handler(_this.current, _this.target);
                });
                this._raf = null;
            }
        }
    }, {
        key: 'update',
        value: function update() {
            var distance = this.target - this.current;
            var attraction = distance * this._acceleration;

            this.applyForce(attraction);

            this.velocity *= this._friction;
            this.current += this.velocity;

            return distance;
        }
    }, {
        key: 'applyForce',
        value: function applyForce(force) {
            this.velocity += force;
        }
    }, {
        key: 'on',
        value: function on(event, handler) {
            if (this._handlers[event]) {
                this._handlers[event].push(handler);
            } else {
                console.warn('Unsupported event ' + event + '.');
            }
        }
    }, {
        key: 'off',
        value: function off(event, handler) {
            if (event != null) {
                if (handler != null) {
                    if (this._handlers[event] && this._handlers[event].filter(function (savedHandler) {
                        return savedHandler === handler;
                    }).length) {
                        var toRemove = this._handlers[event].filter(function (savedHandler) {
                            return savedHandler === handler;
                        })[0];
                        var index = this._handlers[event].indexOf(toRemove);
                        if (index > -1) {
                            this._handlers[event].splice(index, 1);
                        }
                    } else {
                        console.warn('Handler for event ' + event + ' no found.');
                    }
                } else {
                    this._handlers[event] = [];
                }
            } else {
                this._handlers = {};
            }
        }
    }, {
        key: 'target',
        set: function set(num) {
            var _this2 = this;

            this._target = num;
            if (!this._raf) {
                this._handlers['start'].forEach(function (handler) {
                    return handler(_this2.current, _this2.target);
                });
                this.animate();
            }
            return this._target;
        },
        get: function get() {
            return this._target;
        }
    }]);

    return Kinet;
}();

exports.default = Kinet;